#ifndef DRAW_GAME_H_
#define DRAW_GAME_H_

#include <kos.h>
#include <map>
#include <vector>
#include <string>
#include "Video.h"
#include "Utility.h"
#include "Sprite.h"
#include "Player.h"
#include "Enemy.h"
#include "Laser.h"
#include "Level.h"
#include "GameState.h"
#include "Menu.h"

void drawTitleMenu(pvr_ptr_t &titleScreenImage, pvr_ptr_t &cursorImage, Menu &titleMenu);
void drawOptionsMenu(pvr_ptr_t &optionsScreenImage, pvr_ptr_t &cursorImage, Menu &optionsMenu);
void drawCreditsMenu(pvr_ptr_t &creditsImage);
void drawStory(std::vector<pvr_ptr_t> &storyImages, int index);
void drawGame(const GameState &gameState,
		pvr_ptr_t &backgroundImage,
		int backgroundIndex,
		std::map<const std::string, pvr_ptr_t> &images,
		const std::vector<Player> &players,
		const std::vector<Enemy> &enemies,
		const std::vector<Laser> &lasers,
		const std::vector<Level> &levels);
void drawGameOver(pvr_ptr_t &gameOverImage);
void drawWin(pvr_ptr_t &winImage);
template <typename T> void drawSprites(std::map<const std::string, pvr_ptr_t> images,
		std::vector<T> sprites);

template <typename T>
void drawSprites(std::map<const std::string, pvr_ptr_t> images,
		std::vector<T> sprites)
{
	for (typename std::vector<T>::iterator it = sprites.begin(); it != sprites.end(); ++it)
    {
		if(it->getDead() == true)
			continue;

    	float x;
    	float y;
    	float z;
    	float width;
    	float height;

    	it->getCoordinates(x, y, z);
    	it->getImageDimensions(width, height);
    	drawImage(images[it->getImagePath()], x, y, z, width, height);
    }
}

#endif
