#ifndef GAME_STATE_H_
#define GAME_STATE_H_

#include <kos.h>
#include <dc/sound/sfxmgr.h>
#include <map>
#include <vector>
#include <string>
#include "Player.h"
#include "Laser.h"
#include "Enemy.h"
#include "Level.h"
#include "Path.h"
#include "InputState.h"
#include "Init.h"

class GameState
{
private:
	int numberOfPlayers;
	bool paused;
	int currentLevel;
	int maxLevels;
	int screen;
	bool stateChanged;
public:
	GameState();
	GameState(int initialNumberOfPlayers, bool initialPaused, int initialCurrentLevel, int initialMaxLevels);
	void setNumberOfPlayers(int newNumberOfPlayers);
	int getNumberOfPlayers() const;
	void setPaused(bool newPaused);
	bool getPaused() const;
	void nextLevel();
	int getCurrentLevel() const;
	void setMaxLevels(int newMaxLevels);
	int getScreen() const;
	void setScreen(int newScreen,
				pvr_ptr_t &titleScreenImage,
				pvr_ptr_t &optionsScreenImage,
				pvr_ptr_t &cursorImage,
				pvr_ptr_t &gameOverImage,
				pvr_ptr_t &backgroundImage,
				pvr_ptr_t &creditsImage,
				pvr_ptr_t &winImage,
				sfxhnd_t &laserSound,
				std::vector<pvr_ptr_t> &storyImages,
				InputState &inputState,
				std::map<const std::string, pvr_ptr_t> &images,
				std::vector<Player> &players,
				std::vector<Laser> &lasers,
				std::vector<Enemy> &enemies,
				std::vector<Level> &levels,
				std::vector<Path> &paths);
	bool getStateChanged(); //Once this is called, stateChanged will be reset
	void reinitializeGameState();
};

#endif
