#ifndef INPUT_H_
#define INPUT_H_

#include <kos.h>
#include <dc/sound/sfxmgr.h>
#include <map>
#include <string>
#include <vector>
#include "InputState.h"
#include "Player.h"
#include "Enemy.h"
#include "Laser.h"
#include "Path.h"
#include "Level.h"
#include "Menu.h"
#include "Globals.h"
#include "GameState.h"
#include "StoryManager.h"

class GameState;

void getInput(InputState *inputState, GameState *gameState);
void processTitleScreenInput(InputState &inputState, GameState &gameState, Menu &titleMenu, pvr_ptr_t &titleScreenImage, pvr_ptr_t &optionsScreenImage, pvr_ptr_t &cursorImage, pvr_ptr_t &gameOverImage, pvr_ptr_t &backgroundImage, pvr_ptr_t &creditsImage, pvr_ptr_t &winImage, sfxhnd_t &laserSound, std::vector<pvr_ptr_t> &storyImages, std::map<const std::string, pvr_ptr_t> &images, std::vector<Player> &players, std::vector<Laser> &lasers, std::vector<Enemy> &enemies, std::vector<Level> &levels, std::vector<Path> &paths);
void processOptionsScreenInput(InputState &inputState, GameState &gameState, Menu &optionsMenu, pvr_ptr_t &titleScreenImage, pvr_ptr_t &optionsScreenImage, pvr_ptr_t &cursorImage, pvr_ptr_t &gameOverImage, pvr_ptr_t &backgroundImage, pvr_ptr_t &creditsImage, pvr_ptr_t &winImage, sfxhnd_t &laserSound, std::vector<pvr_ptr_t> &storyImages, std::map<const std::string, pvr_ptr_t > &images, std::vector<Player> &players, std::vector<Laser> &lasers, std::vector<Enemy> &enemies, std::vector<Level> &levels, std::vector<Path> &paths);
void processCreditsScreenInput(InputState &inputState, GameState &gameState, Menu &creditsMenu, pvr_ptr_t &titleScreenImage, pvr_ptr_t &optionsScreenImage, pvr_ptr_t &cursorImage, pvr_ptr_t &gameOverImage, pvr_ptr_t &backgroundImage, pvr_ptr_t &creditsImage, pvr_ptr_t &winImage, sfxhnd_t &laserSound, std::vector<pvr_ptr_t> &storyImages, std::map<const std::string, pvr_ptr_t > &images, std::vector<Player> &players, std::vector<Laser> &lasers, std::vector<Enemy> &enemies, std::vector<Level> &levels, std::vector<Path> &paths);
void processStoryInput(InputState &inputState, GameState &gameState, StoryManager &storyManager, pvr_ptr_t &titleScreenImage, pvr_ptr_t &optionsScreenImage, pvr_ptr_t &cursorImage, pvr_ptr_t &gameOverImage, pvr_ptr_t &backgroundImage, pvr_ptr_t &creditsImage, pvr_ptr_t &winImage, sfxhnd_t &laserSound, std::vector<pvr_ptr_t> &storyImages, std::map<const std::string, pvr_ptr_t > &images, std::vector<Player> &players, std::vector<Laser> &lasers, std::vector<Enemy> &enemies, std::vector<Level> &levels, std::vector<Path> &paths);
void processInput(int numberOfTicks, InputState &inputState, GameState &gameState, Menu &optionsMenu, pvr_ptr_t &titleScreenImage, pvr_ptr_t &optionsScreenImage, pvr_ptr_t &cursorImage, pvr_ptr_t &gameOverImage, pvr_ptr_t &backgroundImage, pvr_ptr_t &creditsImage, pvr_ptr_t &winImage, sfxhnd_t &laserSound, std::vector<pvr_ptr_t> &storyImages, std::map<const std::string, pvr_ptr_t > &images, std::vector<Player> &players, std::vector<Laser> &lasers, std::vector<Enemy> &enemies, std::vector<Level> &levels, std::vector<Path> &paths);
void resetNumberOfTicks(InputState *inputState);

#endif
