#ifndef SPRITE_H_
#define SPRITE_H_

#include <vector>
#include <string>
#include "CollisionDetection.h"
#include "Globals.h"

class Sprite
{
protected:
	std::string imagePath;
	float imageWidth;
	float imageHeight;
	float x;
	float y;
	float z;
	float width;
	float height;
	bool dead;

protected:
	bool checkWallCollisions();

public:
	Sprite();
	Sprite(std::string initialImagePath, float initialImageWidth, float initialImageHeight, float initialX, float initialY, float initialZ,  float initialWidth, float initialHeight);
	Sprite(const Sprite& sp);
	void setImagePath(std::string newImagePath);
	std::string getImagePath() const;
	void setCoordinates(float newX, float newY, float newZ);
	void getCoordinates(float &curX, float &curY, float &curZ) const;
	float getZ() const;
	void setDimensions(float newWidth, float newHeight);
	void getDimensions(float &curWidth, float &curHeight) const;
	void getImageDimensions(float &curImageWidth, float &curImageHeight) const;
	void setDead(bool newRemove);
	bool getDead() const;
	template <typename T> int checkSpriteCollisions(std::vector<T> &sprites);
	template <typename T> int checkLaserCollisions(bool isPlayerLaser, std::vector<T> &sprites);
	bool operator==(const Sprite& sp) const;
};

template <typename T>
int Sprite::checkSpriteCollisions(std::vector<T> &sprites)
{
	if (dead)
		return NO_SPRITE;

	float otherSpriteX;
	float otherSpriteY;
	float otherSpriteZ;
	float otherSpriteWidth;
	float otherSpriteHeight;

	for (int i = 0; i < (signed) sprites.size(); ++i)
	{
		if (sprites[i] == *this)
			continue;

		if (sprites[i].getDead())
			continue;

		sprites[i].getCoordinates(otherSpriteX, otherSpriteY, otherSpriteZ);
		sprites[i].getDimensions(otherSpriteWidth, otherSpriteHeight);
		BoundingBox b1(x, y, x + width, y + height);
		BoundingBox b2(otherSpriteX, otherSpriteY,
				otherSpriteX + otherSpriteWidth,
				otherSpriteY + otherSpriteHeight);

		if (RectangularCollisionDetection(b1, b2))
			return i;
	}

	return NO_SPRITE;
}

template <typename T>
int Sprite::checkLaserCollisions(bool isPlayerLaser, std::vector<T> &sprites)
{
	if (dead)
		return NO_SPRITE;

	float otherSpriteX;
	float otherSpriteY;
	float otherSpriteZ;
	float otherSpriteWidth;
	float otherSpriteHeight;

	for (int i = 0; i < (signed) sprites.size(); ++i)
	{
		if (sprites[i] == *this)
			continue;

		if (sprites[i].getPlayerLaser() == isPlayerLaser)
			continue;

		if (sprites[i].getDead())
				continue;

		sprites[i].getCoordinates(otherSpriteX, otherSpriteY, otherSpriteZ);
		sprites[i].getDimensions(otherSpriteWidth, otherSpriteHeight);
		BoundingBox b1(x, y, x + width, y + height);
		BoundingBox b2(otherSpriteX, otherSpriteY,
				otherSpriteX + otherSpriteWidth,
				otherSpriteY + otherSpriteHeight);

		if (RectangularCollisionDetection(b1, b2))
						return i;
	}

	return NO_SPRITE;
}

#endif
