#include "../HeaderFiles/Globals.h"
#include "../HeaderFiles/DrawGame.h"
#include "../HeaderFiles/Utility.h"

void drawTitleMenu(pvr_ptr_t &titleScreenImage, pvr_ptr_t &cursorImage, Menu &titleMenu)
{
	pvr_wait_ready();
	pvr_scene_begin();
	pvr_list_begin(PVR_LIST_TR_POLY);

	drawImage(titleScreenImage, TITLE_SCREEN_STARTING_X, TITLE_SCREEN_STARTING_Y, TITLE_SCREEN_STARTING_Z, SPRITE_WIDTH_TITLE_SCREEN, SPRITE_HEIGHT_TITLE_SCREEN);
	drawImage(cursorImage, CURSOR_STARTING_X, CURSOR_STARTING_Y + titleMenu.getCursorIndex() * CURSOR_Y_VELOCITY, CURSOR_STARTING_Z, SPRITE_WIDTH_CURSOR, SPRITE_HEIGHT_CURSOR);

	pvr_list_finish();
	pvr_scene_finish();
}

void drawOptionsMenu(pvr_ptr_t &optionsScreenImage, pvr_ptr_t &cursorImage, Menu &optionsMenu)
{
	pvr_wait_ready();
	pvr_scene_begin();
	pvr_list_begin(PVR_LIST_TR_POLY);

	drawImage(optionsScreenImage, OPTIONS_SCREEN_STARTING_X, OPTIONS_SCREEN_STARTING_Y, OPTIONS_SCREEN_STARTING_Z, SPRITE_WIDTH_OPTIONS_SCREEN, SPRITE_HEIGHT_OPTIONS_SCREEN);
	drawImage(cursorImage, CURSOR_STARTING_X, CURSOR_STARTING_Y + optionsMenu.getCursorIndex() * CURSOR_Y_VELOCITY, CURSOR_STARTING_Z, SPRITE_WIDTH_CURSOR, SPRITE_HEIGHT_CURSOR);

	pvr_list_finish();
	pvr_scene_finish();
}

void drawCreditsMenu(pvr_ptr_t &creditsImage)
{
	pvr_wait_ready();
	pvr_scene_begin();
	pvr_list_begin(PVR_LIST_TR_POLY);

	drawImage(creditsImage, CREDITS_SCREEN_STARTING_X, CREDITS_SCREEN_STARTING_Y, CREDITS_SCREEN_STARTING_Z, SPRITE_WIDTH_CREDITS_SCREEN, SPRITE_HEIGHT_CREDITS_SCREEN);

	pvr_list_finish();
	pvr_scene_finish();
}

void drawStory(std::vector<pvr_ptr_t> &storyImages, int index)
{
	pvr_wait_ready();
	pvr_scene_begin();
	pvr_list_begin(PVR_LIST_TR_POLY);

	drawImage(storyImages[index], STORY_STARTING_X, STORY_STARTING_Y, STORY_STARTING_Z, IMAGE_WIDTH_STORY, IMAGE_HEIGHT_STORY);

	pvr_list_finish();
	pvr_scene_finish();
}

void drawGameOver(pvr_ptr_t &gameOverImage)
{
	pvr_wait_ready();
	pvr_scene_begin();
	pvr_list_begin(PVR_LIST_TR_POLY);

	drawImage(gameOverImage, GAME_OVER_SCREEN_STARTING_X, GAME_OVER_SCREEN_STARTING_Y, GAME_OVER_SCREEN_STARTING_Z, SPRITE_WIDTH_GAME_OVER_SCREEN, SPRITE_HEIGHT_GAME_OVER_SCREEN);

	pvr_list_finish();
	pvr_scene_finish();
}

void drawWin(pvr_ptr_t &winImage)
{
	pvr_wait_ready();
	pvr_scene_begin();
	pvr_list_begin(PVR_LIST_TR_POLY);

	drawImage(winImage, WIN_SCREEN_STARTING_X, WIN_SCREEN_STARTING_Y, WIN_SCREEN_STARTING_Z, SPRITE_WIDTH_WIN_SCREEN, SPRITE_HEIGHT_WIN_SCREEN);

	pvr_list_finish();
	pvr_scene_finish();
}

void drawGame(const GameState &gameState,
		pvr_ptr_t &backgroundImage,
		int backgroundIndex,
		std::map<const std::string, pvr_ptr_t> &images,
		const std::vector<Player> &players,
		const std::vector<Enemy> &enemies,
		const std::vector<Laser> &lasers,
		const std::vector<Level> &levels)
{
    pvr_wait_ready();
    pvr_scene_begin();
    pvr_list_begin(PVR_LIST_TR_POLY);

    if (backgroundIndex > NUMBER_OF_BACKGROUNDS - 1)
    	backgroundIndex = NUMBER_OF_BACKGROUNDS - 1;

    drawImage(backgroundImage, BACKGROUND_STARTING_X[backgroundIndex], BACKGROUND_STARTING_Y[backgroundIndex], BACKGROUND_STARTING_Z[backgroundIndex], SPRITE_WIDTH_BACKGROUND[backgroundIndex], SPRITE_HEIGHT_BACKGROUND[backgroundIndex]);

    if (!levels[gameState.getCurrentLevel()].getLevelStarted())
    	drawImage(images[IMAGE_PATH_GET_READY], GET_READY_STARTING_X, GET_READY_STARTING_Y, GET_READY_STARTING_Z, IMAGE_WIDTH_GET_READY, IMAGE_HEIGHT_GET_READY);

    drawSprites(images, players);
    drawSprites(images, enemies);
    drawSprites(images, lasers);

    if (gameState.getPaused())
		drawImage(images[IMAGE_PATH_PAUSE], PAUSE_STARTING_X, PAUSE_STARTING_Y, PAUSE_STARTING_Z, IMAGE_WIDTH_PAUSE, IMAGE_HEIGHT_PAUSE);

    //Player Status

    drawImage(images[IMAGE_PATH_PLAYER], PLAYER1_STARTING_X, PLAYER1_STARTING_Y, PLAYER1_STARTING_Z, IMAGE_WIDTH_PLAYER, IMAGE_HEIGHT_PLAYER);
    drawImage(images[IMAGE_PATH_LIVES], LIVES1_STARTING_X, LIVES1_STARTING_Y, LIVES1_STARTING_Z, IMAGE_WIDTH_LIVES, IMAGE_HEIGHT_LIVES);
    drawImage(images[IMAGE_PATH_NUMBER_1], PLAYER1_STARTING_X + IMAGE_WIDTH_PLAYER + TEXT_PADDING, PLAYER1_STARTING_Y + PLAYER_NUMBER_Y_OFFSET, PLAYER1_STARTING_Z, IMAGE_WIDTH_NUMBER_1, IMAGE_HEIGHT_NUMBER_1);

    char index[8];
    integerToString(players[0].getLives(), index);
    drawImage(images[IMAGE_PATH_NUMBER + index + ".png"], PLAYER1_STARTING_X + IMAGE_WIDTH_PLAYER + TEXT_PADDING, LIVES1_STARTING_Y + PLAYER_NUMBER_Y_OFFSET + LIVES_NUMBER_Y_OFFSET, PLAYER1_STARTING_Z, IMAGE_WIDTH_NUMBER_1, IMAGE_HEIGHT_NUMBER_1);

    if (gameState.getNumberOfPlayers() > 1)
    {
    	drawImage(images[IMAGE_PATH_PLAYER], PLAYER2_STARTING_X, PLAYER2_STARTING_Y, PLAYER2_STARTING_Z, IMAGE_WIDTH_PLAYER, IMAGE_HEIGHT_PLAYER);
        drawImage(images[IMAGE_PATH_NUMBER_2], PLAYER2_STARTING_X + IMAGE_WIDTH_PLAYER + TEXT_PADDING, PLAYER2_STARTING_Y + PLAYER_NUMBER_Y_OFFSET, PLAYER2_STARTING_Z, IMAGE_WIDTH_NUMBER_2, IMAGE_HEIGHT_NUMBER_2);
        drawImage(images[IMAGE_PATH_LIVES], LIVES2_STARTING_X, LIVES2_STARTING_Y, LIVES2_STARTING_Z, IMAGE_WIDTH_LIVES, IMAGE_HEIGHT_LIVES);

        integerToString(players[1].getLives(), index);
        //NOTE: I'm using the dimensions of NUMBER_1 because they're all the same
        drawImage(images[IMAGE_PATH_NUMBER + index + ".png"], PLAYER2_STARTING_X + IMAGE_WIDTH_PLAYER + TEXT_PADDING, LIVES2_STARTING_Y + PLAYER_NUMBER_Y_OFFSET + LIVES_NUMBER_Y_OFFSET, PLAYER2_STARTING_Z, IMAGE_WIDTH_NUMBER_1, IMAGE_HEIGHT_NUMBER_1);
    }

    if (gameState.getNumberOfPlayers()  > 2)
    {
		drawImage(images[IMAGE_PATH_NUMBER_3], PLAYER3_STARTING_X + IMAGE_WIDTH_PLAYER + TEXT_PADDING, PLAYER3_STARTING_Y + PLAYER_NUMBER_Y_OFFSET, PLAYER3_STARTING_Z, IMAGE_WIDTH_NUMBER_3, IMAGE_HEIGHT_NUMBER_3);
		drawImage(images[IMAGE_PATH_LIVES], LIVES3_STARTING_X, LIVES3_STARTING_Y, LIVES3_STARTING_Z, IMAGE_WIDTH_LIVES, IMAGE_HEIGHT_LIVES);
		drawImage(images[IMAGE_PATH_PLAYER], PLAYER3_STARTING_X, PLAYER3_STARTING_Y, PLAYER3_STARTING_Z, IMAGE_WIDTH_PLAYER, IMAGE_HEIGHT_PLAYER);

		integerToString(players[2].getLives(), index);
		//NOTE: I'm using the dimensions of NUMBER_1 because they're all the same
		drawImage(images[IMAGE_PATH_NUMBER + index + ".png"], PLAYER3_STARTING_X + IMAGE_WIDTH_PLAYER + TEXT_PADDING, LIVES3_STARTING_Y + PLAYER_NUMBER_Y_OFFSET + LIVES_NUMBER_Y_OFFSET, PLAYER3_STARTING_Z, IMAGE_WIDTH_NUMBER_1, IMAGE_HEIGHT_NUMBER_1);
    }

    if (gameState.getNumberOfPlayers()  > 3)
    {
    	drawImage(images[IMAGE_PATH_PLAYER], PLAYER4_STARTING_X, PLAYER4_STARTING_Y, PLAYER4_STARTING_Z, IMAGE_WIDTH_PLAYER, IMAGE_HEIGHT_PLAYER);
    	drawImage(images[IMAGE_PATH_LIVES], LIVES4_STARTING_X, LIVES4_STARTING_Y, LIVES4_STARTING_Z, IMAGE_WIDTH_LIVES, IMAGE_HEIGHT_LIVES);
		drawImage(images[IMAGE_PATH_NUMBER_4], PLAYER4_STARTING_X + IMAGE_WIDTH_PLAYER + TEXT_PADDING, PLAYER4_STARTING_Y + PLAYER_NUMBER_Y_OFFSET, PLAYER4_STARTING_Z, IMAGE_WIDTH_NUMBER_4, IMAGE_HEIGHT_NUMBER_4);

		integerToString(players[3].getLives(), index);
		//NOTE: I'm using the dimensions of NUMBER_1 because they're all the same
		drawImage(images[IMAGE_PATH_NUMBER + index + ".png"], PLAYER4_STARTING_X + IMAGE_WIDTH_PLAYER + TEXT_PADDING, LIVES4_STARTING_Y + PLAYER_NUMBER_Y_OFFSET + LIVES_NUMBER_Y_OFFSET, PLAYER4_STARTING_Z, IMAGE_WIDTH_NUMBER_1, IMAGE_HEIGHT_NUMBER_1);
    }

    pvr_list_finish();
    pvr_scene_finish();
}
