#include "../HeaderFiles/Enemy.h"
#include "../HeaderFiles/Utility.h"
#include "../HeaderFiles/Sprite.h"
#include "../HeaderFiles/Player.h"
#include "../HeaderFiles/Laser.h"

Enemy::Enemy()
{
	currentPointIndex = 0;
}

Enemy::Enemy(std::string initialImagePath, float initialImageWidth, float initialImageHeight, float initalX, float initialY, float initialZ, float initialWidth, float initialHeight) :
Sprite(initialImagePath, initialImageWidth, initialImageHeight, initalX,  initialY, initialZ, initialWidth, initialHeight),
currentPointIndex(0),
isFinishedOnPath(false)
{
}

void Enemy::move(float xVelocity, float yVelocity, std::vector<Player> &players, std::vector<Laser> &lasers)
{
	if (dead)
		return;

	//Not going to force enemies to stay in bounds
	x += xVelocity;
	y += yVelocity;

	int playerCollisionsIndex = checkSpriteCollisions(players);
	if (playerCollisionsIndex != NO_SPRITE)
	{
		if (!players[playerCollisionsIndex].invincible())
		{
			players[playerCollisionsIndex].setDead(true);
			setDead(true);
		}
	}

	//Special case with lasers because player originated lasers should not kill the player
	int laserCollisionsIndex = checkLaserCollisions(ENEMY_LASER, lasers);
	if (laserCollisionsIndex != NO_SPRITE)
	{
		lasers[laserCollisionsIndex].setDead(true);
		setDead(true);
	}
}

void Enemy::moveToPoint(Point p, std::vector<Player> &players, std::vector<Laser> &lasers)
{
	if (dead)
		return;

	float xVelocity = 0;
	float yVelocity = 0;

	float absDeltaX = abs(p.x - x);
	float absDeltaY = abs(p.y - y);
	float higherDenominator;

	if (absDeltaX / ENEMY_MAX_VELOCITY > absDeltaY / ENEMY_MAX_VELOCITY)
		higherDenominator = absDeltaX / ENEMY_MAX_VELOCITY;
	else
		higherDenominator = absDeltaY / ENEMY_MAX_VELOCITY;

	if (higherDenominator <= 1.0f)
	{
		xVelocity = p.x - x;
		yVelocity = p.y - y;
	}
	else
	{
		xVelocity = (p.x - x) / higherDenominator;
		yVelocity = (p.y - y) / higherDenominator;
	}

	move(xVelocity, yVelocity, players, lasers);
}

void Enemy::moveOnPath(Path &path, std::vector<Player> &players, std::vector<Laser> &lasers)
{
	if (dead)
		return;

	Point p = path.getPoint(currentPointIndex);

	if (x == p.x && y == p.y)
			p = path.getPoint(++currentPointIndex);

	moveToPoint(p, players, lasers);
}

bool Enemy::finishedPath(Path currentPath)
{
	if (isFinishedOnPath)
		return true;

	if(currentPath.isDone(currentPointIndex))
	{
		isFinishedOnPath = true;
		return true;
	}

	return false;
}

void Enemy::shoot(float xVelocity, float yVelocity, std::vector<Laser> &lasers, sfxhnd_t &laserSound)
{
	if (dead)
		return;

	snd_sfx_play(laserSound, 255, 128); //Max volume no pan

	bool laserAdded = false;

	for (std::vector<Laser>::iterator it = lasers.begin();
			it != lasers.end(); ++it)
	{
		if(it->getDead() == true)
		{
			it->setCoordinates(x + SPRITE_ENEMY_WIDTH / 2, y + SPRITE_ENEMY_HEIGHT + 1, it->getZ());
			it->setDead(false);
			it->setVelocities(xVelocity, yVelocity);
			it->setPlayerLaser(ENEMY_LASER);
			laserAdded = true;
			break;
		}
	}

	if (laserAdded == false)
	{
		lasers.push_back(Laser(IMAGE_PATH_LASER, IMAGE_WIDTH_LASER, IMAGE_HEIGHT_LASER, x + SPRITE_ENEMY_WIDTH / 2, y + SPRITE_ENEMY_HEIGHT + 1, LASER_STARTING_Z + lasers.size(), SPRITE_LASER_WIDTH, SPRITE_LASER_HEIGHT, xVelocity, yVelocity, ENEMY_LASER));
		laserAdded = true;
	}
}
