#include "../HeaderFiles/GameState.h"

GameState::GameState()
: numberOfPlayers(1),
paused(false),
currentLevel(0),
maxLevels(0),
screen(NULL_SCREEN),
stateChanged(false)
{
}

GameState::GameState(int initialNumberOfPlayers, bool initialPaused, int initialCurrentLevel, int initialMaxLevels)
: numberOfPlayers(initialNumberOfPlayers),
  paused(false),
  currentLevel(initialCurrentLevel),
  maxLevels(initialMaxLevels),
  screen(NULL_SCREEN),
  stateChanged(false)
{
}

void GameState::setNumberOfPlayers(int newNumberOfPlayers)
{
	numberOfPlayers = newNumberOfPlayers;
}

int GameState::getNumberOfPlayers() const
{
	return numberOfPlayers;
}

void GameState::setPaused(bool newPaused)
{
	paused = newPaused;
}

bool GameState::getPaused() const
{
	return paused;
}

void GameState::nextLevel()
{
	if (currentLevel < maxLevels)
		++currentLevel;
}

int GameState::getCurrentLevel() const
{
	return currentLevel;
}

void GameState::setMaxLevels(int newMaxLevels)
{
	maxLevels = newMaxLevels;
}

int GameState::getScreen() const
{
	return screen;
}

void GameState::setScreen(int newScreen,
			pvr_ptr_t &titleScreenImage,
			pvr_ptr_t &optionsScreenImage,
			pvr_ptr_t &cursorImage,
			pvr_ptr_t &gameOverImage,
			pvr_ptr_t &backgroundImage,
			pvr_ptr_t &creditsImage,
			pvr_ptr_t &winImage,
			sfxhnd_t &laserSound,
			std::vector<pvr_ptr_t> &storyImages,
			InputState &inputState,
			std::map<const std::string, pvr_ptr_t> &images,
			std::vector<Player> &players,
			std::vector<Laser> &lasers,
			std::vector<Enemy> &enemies,
			std::vector<Level> &levels,
			std::vector<Path> &paths)
{
	if (screen != NULL_SCREEN)
	{
		switch (screen)
		{
		case TITLE_SCREEN:
			exitTitleScreen(titleScreenImage, cursorImage);
			break;
		case OPTIONS_SCREEN:
			exitOptionsScreen(optionsScreenImage, cursorImage);
			break;
		case CREDITS_SCREEN:
			exitCreditsScreen(creditsImage);
			break;
		case STORY_SCREEN:
			exitStoryScreen(storyImages);
			break;
		case GAME_SCREEN:
			if (newScreen != STORY_SCREEN) //Bad hack: I don't want the game to reinitialize when the screen cuts to story
				exitGameScreen(*this, inputState, backgroundImage, images, players, lasers, enemies, levels, paths);
			break;
		case SURVIVAL_SCREEN:
			exitGameScreen(*this, inputState, backgroundImage, images, players, lasers, enemies, levels, paths);
			break;
		case GAME_OVER_SCREEN:
			exitGameOverScreen(gameOverImage);
			break;
		case WIN_SCREEN:
			exitWinScreen(winImage);
		}
	}

	screen = newScreen;
	stateChanged = true;

	switch (screen)
	{
	case TITLE_SCREEN:
		initTitleScreen(titleScreenImage, cursorImage);
		break;
	case OPTIONS_SCREEN:
		initOptionsScreen(optionsScreenImage, cursorImage);
		break;
	case CREDITS_SCREEN:
		initCreditsScreen(creditsImage);
		break;
	case STORY_SCREEN:
		initStoryScreen(storyImages, getCurrentLevel());
		break;
	case GAME_SCREEN:
		initGame(inputState, backgroundImage, laserSound, images, players, lasers, enemies, levels, paths, *this);
		break;
	case SURVIVAL_SCREEN:
		initSurvival(inputState, backgroundImage, laserSound, images, players, lasers, enemies, levels, paths, *this);
		break;
	case GAME_OVER_SCREEN:
		initGameOverScreen(gameOverImage);
		break;
	case WIN_SCREEN:
		initWinScreen(winImage);
		break;
	}
}

//stateChanged resets when this function is called
bool GameState::getStateChanged()
{
	bool tmp = stateChanged;

	if (stateChanged == true)
		stateChanged = false;

	return tmp;
}

void GameState::reinitializeGameState()
{
	currentLevel = 0;
	paused = false;
}
