#include <oggvorbis/sndoggvorbis.h>
#include "../HeaderFiles/Init.h"

void initTitleScreen(pvr_ptr_t &titleScreenImage, pvr_ptr_t &cursorImage)
{
	sndoggvorbis_stop();
	snd_sfx_unload_all();
	loadImage(&titleScreenImage, IMAGE_PATH_TITLE_SCREEN,
		IMAGE_WIDTH_TITLE_SCREEN, IMAGE_HEIGHT_TITLE_SCREEN);

	if(!titleScreenImage)
		exit(EXIT_FAILURE);

	loadImage(&cursorImage, IMAGE_PATH_CURSOR,
		IMAGE_WIDTH_CURSOR, IMAGE_HEIGHT_CURSOR);

	if(!cursorImage)
		exit(EXIT_FAILURE);
}

void initOptionsScreen(pvr_ptr_t &optionsScreenImage, pvr_ptr_t &cursorImage)
{
	sndoggvorbis_stop();
	snd_sfx_unload_all();
	loadImage(&optionsScreenImage, IMAGE_PATH_OPTIONS_SCREEN,
		IMAGE_WIDTH_OPTIONS_SCREEN, IMAGE_HEIGHT_OPTIONS_SCREEN);

	if(!optionsScreenImage)
		exit(EXIT_FAILURE);

	loadImage(&cursorImage, IMAGE_PATH_CURSOR,
		IMAGE_WIDTH_CURSOR, IMAGE_HEIGHT_CURSOR);

	if(!cursorImage)
		exit(EXIT_FAILURE);
}

void initCreditsScreen(pvr_ptr_t &creditsImage)
{
	sndoggvorbis_stop();
	snd_sfx_unload_all();
	loadImage(&creditsImage, IMAGE_PATH_CREDITS_SCREEN,
		IMAGE_WIDTH_CREDITS_SCREEN, IMAGE_HEIGHT_CREDITS_SCREEN);

	if(!creditsImage)
		exit(EXIT_FAILURE);
}

void initStoryScreen(std::vector<pvr_ptr_t> &storyImages, int currentLevel)
{
	pvr_ptr_t tmpImage;
	storyImages.clear();

	if (currentLevel == 0)
	{
		for (int i = 0; i < STORY_IMAGES_BEFORE_LEVEL[0]; ++i)
		{
			loadImage(&tmpImage, IMAGE_PATH_STORY[i],
					IMAGE_WIDTH_STORY, IMAGE_HEIGHT_STORY);

			if(!tmpImage)
				exit(EXIT_FAILURE);

			storyImages.push_back(tmpImage);
		}
	}

	if (currentLevel == 1)
	{
		for (int i = STORY_IMAGES_BEFORE_LEVEL[0]; i < STORY_IMAGES_BEFORE_LEVEL[1];
				++i)
		{
			loadImage(&tmpImage, IMAGE_PATH_STORY[i],
					IMAGE_WIDTH_STORY, IMAGE_HEIGHT_STORY);

			if(!tmpImage)
				exit(EXIT_FAILURE);

			storyImages.push_back(tmpImage);
		}
	}

	if (currentLevel == 2)
	{
		for (int i = STORY_IMAGES_BEFORE_LEVEL[1]; i < STORY_IMAGES_BEFORE_LEVEL[2];
				++i)
		{
			loadImage(&tmpImage, IMAGE_PATH_STORY[i],
					IMAGE_WIDTH_STORY, IMAGE_HEIGHT_STORY);

			if(!tmpImage)
				exit(EXIT_FAILURE);

			storyImages.push_back(tmpImage);
		}
	}

	if (currentLevel == 3)
	{
		for (int i = STORY_IMAGES_BEFORE_LEVEL[2]; i < STORY_IMAGES_BEFORE_LEVEL[3];
				++i)
		{
			loadImage(&tmpImage, IMAGE_PATH_STORY[i],
					IMAGE_WIDTH_STORY, IMAGE_HEIGHT_STORY);

			if(!tmpImage)
				exit(EXIT_FAILURE);

			storyImages.push_back(tmpImage);
		}
	}

	if (currentLevel == 4)
	{
		for (int i = STORY_IMAGES_BEFORE_LEVEL[3]; i < STORY_IMAGES_BEFORE_LEVEL[4];
				++i)
		{
			loadImage(&tmpImage, IMAGE_PATH_STORY[i],
					IMAGE_WIDTH_STORY, IMAGE_HEIGHT_STORY);

			if(!tmpImage)
				exit(EXIT_FAILURE);

			storyImages.push_back(tmpImage);
		}
	}

	if (currentLevel == 5)
	{
		for (int i = STORY_IMAGES_BEFORE_LEVEL[4]; i < STORY_IMAGES_BEFORE_LEVEL[5];
				++i)
		{
			loadImage(&tmpImage, IMAGE_PATH_STORY[i],
					IMAGE_WIDTH_STORY, IMAGE_HEIGHT_STORY);

			if(!tmpImage)
				exit(EXIT_FAILURE);

			storyImages.push_back(tmpImage);
		}
	}

	if (currentLevel == 6)
	{
		for (int i = STORY_IMAGES_BEFORE_LEVEL[5]; i < STORY_IMAGES_BEFORE_LEVEL[6];
				++i)
		{
			loadImage(&tmpImage, IMAGE_PATH_STORY[i],
					IMAGE_WIDTH_STORY, IMAGE_HEIGHT_STORY);

			if(!tmpImage)
				exit(EXIT_FAILURE);

			storyImages.push_back(tmpImage);
		}
	}

	if (currentLevel == 7)
	{
		for (int i = STORY_IMAGES_BEFORE_LEVEL[6]; i < STORY_IMAGES_BEFORE_LEVEL[7];
				++i)
		{
			loadImage(&tmpImage, IMAGE_PATH_STORY[i],
					IMAGE_WIDTH_STORY, IMAGE_HEIGHT_STORY);

			if(!tmpImage)
				exit(EXIT_FAILURE);

			storyImages.push_back(tmpImage);
		}
	}

	if (currentLevel == 8)
	{
		for (int i = STORY_IMAGES_BEFORE_LEVEL[7]; i < STORY_IMAGES_BEFORE_LEVEL[8];
				++i)
		{
			loadImage(&tmpImage, IMAGE_PATH_STORY[i],
					IMAGE_WIDTH_STORY, IMAGE_HEIGHT_STORY);

			if(!tmpImage)
				exit(EXIT_FAILURE);

			storyImages.push_back(tmpImage);
		}
	}
}

void initGame(InputState &inputState,
		pvr_ptr_t &backgroundImage,
		sfxhnd_t &laserSound,
		std::map<const std::string, pvr_ptr_t> &images,
		std::vector<Player> &players,
		std::vector<Laser> &lasers,
		std::vector<Enemy> &enemies,
		std::vector<Level> &levels,
		std::vector<Path> &paths,
		GameState &gameState)
{
	if (levels.size() != 0)
		return; //This is a hack to make sure init game is not called twice before exitGame is called

	initGameImages(inputState, backgroundImage, laserSound, images, players, lasers, enemies, levels, paths, gameState);
	createLevels(levels);
	gameState.setMaxLevels(levels.size() - 1);
}

void initSurvival(InputState &inputState,
		pvr_ptr_t &backgroundImage,
		sfxhnd_t &laserSound,
		std::map<const std::string, pvr_ptr_t> &images,
		std::vector<Player> &players,
		std::vector<Laser> &lasers,
		std::vector<Enemy> &enemies,
		std::vector<Level> &levels,
		std::vector<Path> &paths,
		GameState &gameState)
{
	initGameImages(inputState, backgroundImage, laserSound, images, players, lasers, enemies, levels, paths, gameState);
	createSurvival(levels);
	gameState.setMaxLevels(levels.size() - 1);
}

void initGameImages(InputState &inputState,
pvr_ptr_t &backgroundImage,
sfxhnd_t &laserSound,
std::map<const std::string, pvr_ptr_t> &images,
std::vector<Player> &players,
std::vector<Laser> &lasers,
std::vector<Enemy> &enemies,
std::vector<Level> &levels,
std::vector<Path> &paths,
GameState &gameState)
{
	resetNumberOfTicks(&inputState);

	if (!sndoggvorbis_isplaying())
		sndoggvorbis_start(MUSIC_PATH_BUMPING.c_str(), 1);

	laserSound = snd_sfx_load(SOUND_PATH_LASER.c_str());

	loadImage(&backgroundImage, IMAGE_PATH_BACKGROUND[0],
		IMAGE_WIDTH_BACKGROUND[0], IMAGE_HEIGHT_BACKGROUND[0]);

	if(!backgroundImage)
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_PLAYER_SHIP_1], IMAGE_PATH_PLAYER_SHIP_1,
		IMAGE_WIDTH_PLAYER_SHIP, IMAGE_HEIGHT_PLAYER_SHIP);

	if(!images[IMAGE_PATH_PLAYER_SHIP_1])
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_PLAYER_SHIP_2], IMAGE_PATH_PLAYER_SHIP_2,
			IMAGE_WIDTH_PLAYER_SHIP, IMAGE_HEIGHT_PLAYER_SHIP);

	if(!images[IMAGE_PATH_PLAYER_SHIP_2])
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_PLAYER_SHIP_3], IMAGE_PATH_PLAYER_SHIP_3,
			IMAGE_WIDTH_PLAYER_SHIP, IMAGE_HEIGHT_PLAYER_SHIP);

	if(!images[IMAGE_PATH_PLAYER_SHIP_3])
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_PLAYER_SHIP_4], IMAGE_PATH_PLAYER_SHIP_4,
			IMAGE_WIDTH_PLAYER_SHIP, IMAGE_HEIGHT_PLAYER_SHIP);

	if(!images[IMAGE_PATH_PLAYER_SHIP_4])
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_LASER], IMAGE_PATH_LASER,
		IMAGE_WIDTH_LASER, IMAGE_HEIGHT_LASER);

	if(!images[IMAGE_PATH_LASER])
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_UFO], IMAGE_PATH_UFO,
		IMAGE_WIDTH_UFO, IMAGE_HEIGHT_UFO);

	if(!images[IMAGE_PATH_UFO])
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_PAUSE], IMAGE_PATH_PAUSE,
		IMAGE_WIDTH_PAUSE, IMAGE_HEIGHT_PAUSE);

	if(!images[IMAGE_PATH_PAUSE])
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_PLAYER], IMAGE_PATH_PLAYER,
		IMAGE_WIDTH_PLAYER, IMAGE_HEIGHT_PLAYER);

	if(!images[IMAGE_PATH_PLAYER])
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_NUMBER_0], IMAGE_PATH_NUMBER_0,
			IMAGE_WIDTH_NUMBER_0, IMAGE_HEIGHT_NUMBER_0);

	if(!images[IMAGE_PATH_NUMBER_0])
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_NUMBER_1], IMAGE_PATH_NUMBER_1,
		IMAGE_WIDTH_NUMBER_1, IMAGE_HEIGHT_NUMBER_1);

	if(!images[IMAGE_PATH_NUMBER_1])
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_NUMBER_2], IMAGE_PATH_NUMBER_2,
		IMAGE_WIDTH_NUMBER_2, IMAGE_HEIGHT_NUMBER_2);

	if(!images[IMAGE_PATH_NUMBER_2])
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_NUMBER_3], IMAGE_PATH_NUMBER_3,
		IMAGE_WIDTH_NUMBER_3, IMAGE_HEIGHT_NUMBER_3);

	if(!images[IMAGE_PATH_NUMBER_3])
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_NUMBER_4], IMAGE_PATH_NUMBER_4,
		IMAGE_WIDTH_NUMBER_4, IMAGE_HEIGHT_NUMBER_4);

	if(!images[IMAGE_PATH_NUMBER_4])
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_NUMBER_5], IMAGE_PATH_NUMBER_5,
		IMAGE_WIDTH_NUMBER_5, IMAGE_HEIGHT_NUMBER_5);

	if(!images[IMAGE_PATH_NUMBER_5])
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_NUMBER_6], IMAGE_PATH_NUMBER_6,
		IMAGE_WIDTH_NUMBER_6, IMAGE_HEIGHT_NUMBER_6);

	if(!images[IMAGE_PATH_NUMBER_6])
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_NUMBER_7], IMAGE_PATH_NUMBER_7,
		IMAGE_WIDTH_NUMBER_7, IMAGE_HEIGHT_NUMBER_7);

	if(!images[IMAGE_PATH_NUMBER_7])
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_NUMBER_8], IMAGE_PATH_NUMBER_8,
		IMAGE_WIDTH_NUMBER_8, IMAGE_HEIGHT_NUMBER_8);

	if(!images[IMAGE_PATH_NUMBER_8])
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_NUMBER_9], IMAGE_PATH_NUMBER_9,
		IMAGE_WIDTH_NUMBER_9, IMAGE_HEIGHT_NUMBER_9);

	if(!images[IMAGE_PATH_NUMBER_9])
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_LIVES], IMAGE_PATH_LIVES,
			IMAGE_WIDTH_LIVES, IMAGE_HEIGHT_LIVES);

	if(!images[IMAGE_PATH_LIVES])
		exit(EXIT_FAILURE);

	loadImage(&images[IMAGE_PATH_GET_READY], IMAGE_PATH_GET_READY,
		IMAGE_WIDTH_GET_READY, IMAGE_HEIGHT_GET_READY);

	if(!images[IMAGE_PATH_GET_READY])
		exit(EXIT_FAILURE);

	players.push_back(Player(IMAGE_PATH_PLAYER_SHIP_1, IMAGE_WIDTH_PLAYER_SHIP, IMAGE_HEIGHT_PLAYER_SHIP, PLAYER_SHIP1_STARTING_X,
		PLAYER_SHIP1_STARTING_Y, PLAYER_SHIP1_STARTING_Z, SPRITE_PLAYER_WIDTH,
		SPRITE_PLAYER_HEIGHT, INITIAL_LIVES));

	if (gameState.getNumberOfPlayers() > 1)
	{
		players.push_back(Player(IMAGE_PATH_PLAYER_SHIP_2, IMAGE_WIDTH_PLAYER_SHIP, IMAGE_HEIGHT_PLAYER_SHIP, PLAYER_SHIP2_STARTING_X,
				PLAYER_SHIP2_STARTING_Y, PLAYER_SHIP2_STARTING_Z, SPRITE_PLAYER_WIDTH,
				SPRITE_PLAYER_HEIGHT, INITIAL_LIVES));
	}

	if (gameState.getNumberOfPlayers() > 2)
	{
		players.push_back(Player(IMAGE_PATH_PLAYER_SHIP_3, IMAGE_WIDTH_PLAYER_SHIP, IMAGE_HEIGHT_PLAYER_SHIP, PLAYER_SHIP3_STARTING_X,
				PLAYER_SHIP3_STARTING_Y, PLAYER_SHIP3_STARTING_Z, SPRITE_PLAYER_WIDTH,
				SPRITE_PLAYER_HEIGHT, INITIAL_LIVES));
	}

	if (gameState.getNumberOfPlayers() > 3)
	{
		players.push_back(Player(IMAGE_PATH_PLAYER_SHIP_4, IMAGE_WIDTH_PLAYER_SHIP, IMAGE_HEIGHT_PLAYER_SHIP, PLAYER_SHIP4_STARTING_X,
				PLAYER_SHIP4_STARTING_Y, PLAYER_SHIP4_STARTING_Z, SPRITE_PLAYER_WIDTH,
				SPRITE_PLAYER_HEIGHT, INITIAL_LIVES));
	}

	createPaths(paths);
}

void initGameOverScreen(pvr_ptr_t &gameOverImage)
{
	sndoggvorbis_stop();
	snd_sfx_unload_all();

	loadImage(&gameOverImage, IMAGE_PATH_GAME_OVER_SCREEN,
		IMAGE_WIDTH_GAME_OVER_SCREEN, IMAGE_HEIGHT_GAME_OVER_SCREEN);

	if(!gameOverImage)
		exit(EXIT_FAILURE);
}

void initWinScreen(pvr_ptr_t &winImage)
{
	sndoggvorbis_stop();
	snd_sfx_unload_all();

	loadImage(&winImage, IMAGE_PATH_WIN_SCREEN,
		IMAGE_WIDTH_WIN_SCREEN, IMAGE_HEIGHT_WIN_SCREEN);

	if(!winImage)
		exit(EXIT_FAILURE);
}

void exitGameScreen(GameState & gameState,
		InputState &inputState,
		pvr_ptr_t &backgroundImage,
		std::map<const std::string, pvr_ptr_t> &images,
		std::vector<Player> &players,
		std::vector<Laser> &lasers,
		std::vector<Enemy> &enemies,
		std::vector<Level> &levels,
		std::vector<Path> &paths)
{
	for(std::map<const std::string, pvr_ptr_t>::iterator it = images.begin();
			it != images.end(); ++it)
		freeImage(&(it->second));

	freeImage(&backgroundImage);

	inputState.reinitializeInputState();
	gameState.reinitializeGameState();

	players.clear();
	lasers.clear();
	enemies.clear();
	levels.clear();
	paths.clear();
}

void exitTitleScreen(pvr_ptr_t &titleScreenImage, pvr_ptr_t & cursorImage)
{
	freeImage(&titleScreenImage);
	freeImage(&cursorImage);
}

void exitOptionsScreen(pvr_ptr_t &optionsScreenImage, pvr_ptr_t & cursorImage)
{
	freeImage(&optionsScreenImage);
	freeImage(&cursorImage);
}

void exitGameOverScreen(pvr_ptr_t &gameOverImage)
{
	freeImage(&gameOverImage);
}

void exitWinScreen(pvr_ptr_t &winImage)
{
	freeImage(&winImage);
}

void exitCreditsScreen(pvr_ptr_t &creditsImage)
{
	freeImage(&creditsImage);
}

void exitStoryScreen(std::vector<pvr_ptr_t> &storyImages)
{
	for(std::vector<pvr_ptr_t>::iterator it = storyImages.begin();
			it != storyImages.end(); ++it)
		freeImage(&(*it));
}
