#include <oggvorbis/sndoggvorbis.h>
#include "../HeaderFiles/Input.h"

InputState::InputState()
{
	for (int i = 0; i < MAX_PLAYERS; ++i)
	{
		CLeftFireTicks[i] = 0;
		CRightFireTicks[i] = 0;
		CMoveTicks[i] = 0;
	}
}

void InputState::reinitializeInputState()
{
	for (int i = 0; i < MAX_PLAYERS; ++i)
	{
		CUp[i] = false;
		CDown[i] = false;
		CLeft[i] = false;
		CRight[i] = false;
		CJoyY[i] = 0;
		CJoyX[i] = 0;
		CLeftTrigger[i] = 0;
		CRightTrigger[i] = 0;
		CStart[i] = false;
		CA[i] = false;
		CB[i] = false;
		CStartReady[i] = false; //Only tapping the start button should modify the state, thus the need for this variable
		CLeftFireReady[i] = false;
		CRightFireReady[i] = false;
		CLeftFireTicks[i] = 0;
		CRightFireTicks[i] = 0;
		CMoveTicks[i] = 0;
		CATicks[i] = 0;
	}
}

void getInput(InputState *inputState, GameState *gameState)
{
    cont_state_t *state;
    maple_device_t *addr[MAX_PLAYERS];

    for (int i = 0; i < gameState->getNumberOfPlayers() ; ++i)
    {
    	addr[i] = maple_enum_type(i, MAPLE_FUNC_CONTROLLER);
    	state = (cont_state_t *) maple_dev_status(addr[i]);

    	if (state->buttons & CONT_START)
    		inputState->CStart[i] = true;
    	else
    		inputState->CStart[i] = false;

    	if (state->buttons & CONT_A)
    		inputState->CA[i] = true;
		else
			inputState->CA[i] = false;

    	if (state->buttons & CONT_B)
			inputState->CB[i] = true;
		else
			inputState->CB[i] = false;

		if(state->buttons & CONT_DPAD_UP)
			inputState->CUp[i] = true;
		else
			inputState->CUp[i] = false;
	
		if(state->buttons & CONT_DPAD_DOWN)
			inputState->CDown[i] = true;
		else
			inputState->CDown[i] = false;

		if(state->buttons & CONT_DPAD_LEFT)
			inputState->CLeft[i] = true;
		else
			inputState->CLeft[i] = false;

		if(state->buttons & CONT_DPAD_RIGHT)
			inputState->CRight[i] = true;
		else
			inputState->CRight[i] = false;
	
		if (state->joyy < 0 || state->joyy > 0)
		{
			inputState->CUp[i] = false;
			inputState->CDown[i] = false;
			inputState->CJoyY[i] = state->joyy / JOYY_DEGREE_MODIFIER;
		}
		else
			inputState->CJoyY[i] = 0;

		if (state->joyx < 0 || state->joyx > 0)
		{
			inputState->CLeft[i] = false;
			inputState->CRight[i] = false;
			inputState->CJoyX[i] = state->joyy / JOYY_DEGREE_MODIFIER;
		}
		else
			inputState->CJoyX[i] = 0;

		if (state->joyx < 0 || state->joyx > 0)
		{
			inputState->CLeft[i] = false;
			inputState->CRight[i] = false;
			inputState->CJoyX[i] = state->joyx / JOYX_DEGREE_MODIFIER;
		}
		else
			inputState->CJoyX[i] = 0;

		if (state->ltrig > 0)
			inputState->CLeftTrigger[i] = true;
		else
			inputState->CLeftTrigger[i] = false;

		if (state->rtrig > 0)
			inputState->CRightTrigger[i] = true;
		else
			inputState->CRightTrigger[i] = false;
	}
}

void processTitleScreenInput(InputState &inputState, GameState &gameState, Menu &titleMenu, pvr_ptr_t &titleScreenImage, pvr_ptr_t &optionsScreenImage, pvr_ptr_t &cursorImage, pvr_ptr_t &gameOverImage, pvr_ptr_t &backgroundImage, pvr_ptr_t &creditsImage, pvr_ptr_t &winImage, sfxhnd_t &laserSound, std::vector<pvr_ptr_t> &storyImages, std::map<const std::string, pvr_ptr_t> &images, std::vector<Player> &players, std::vector<Laser> &lasers, std::vector<Enemy> &enemies, std::vector<Level> &levels, std::vector<Path> &paths)
{
	//Only allow controller 1 to input

	//In case the number of ticks overflows
	if (titleMenu.getNumberOfTicks() - inputState.CMoveTicks[0] < 0)
		inputState.CMoveTicks[0] = titleMenu.getNumberOfTicks();
	if (titleMenu.getNumberOfTicks() - inputState.CATicks[0] < 0)
		inputState.CATicks[0] = titleMenu.getNumberOfTicks();

	if (inputState.CA[0])
	{
		if (titleMenu.getNumberOfTicks() - inputState.CATicks[0] > A_BUTTON_TICK_THRESHOLD)
		{
			switch (titleMenu.getCursorIndex())
			{
			case START_ENTRY:
				gameState.setScreen(STORY_SCREEN, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, inputState, images, players, lasers, enemies, levels, paths);
				break;
			case SURVIVAL_ENTRY:
				gameState.setScreen(SURVIVAL_SCREEN, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, inputState, images, players, lasers, enemies, levels, paths);
				break;
			case OPTIONS_ENTRY:
				gameState.setScreen(OPTIONS_SCREEN, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, inputState, images, players, lasers, enemies, levels, paths);
				break;
			case CREDITS_ENTRY:
				gameState.setScreen(CREDITS_SCREEN, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, inputState, images, players, lasers, enemies, levels, paths);
				break;
			}
			inputState.CATicks[0] = titleMenu.getNumberOfTicks();
		}
	}
	else
		inputState.CATicks[0] = -A_BUTTON_TICK_THRESHOLD; //If A is not pressed down, the game should respond immediately when it is pressed down

	if (inputState.CUp[0])
	{
		if (titleMenu.getNumberOfTicks() - inputState.CMoveTicks[0] > MOVE_TICK_THRESHOLD)
		{
			titleMenu.moveCursor(UP);
			inputState.CMoveTicks[0] = titleMenu.getNumberOfTicks();
		}
	}
	else if (inputState.CDown[0])
	{
		if (titleMenu.getNumberOfTicks() - inputState.CMoveTicks[0] > MOVE_TICK_THRESHOLD)
		{
			titleMenu.moveCursor(DOWN);
			inputState.CMoveTicks[0] = titleMenu.getNumberOfTicks();
		}
	}

	if (inputState.CJoyY[0] < 0)
	{
		if (titleMenu.getNumberOfTicks() - inputState.CMoveTicks[0] > MOVE_TICK_THRESHOLD)
		{
			titleMenu.moveCursor(UP);
			inputState.CMoveTicks[0] = titleMenu.getNumberOfTicks();
		}
	}

	else if (inputState.CJoyY[0] > 0)
	{
		if (titleMenu.getNumberOfTicks() - inputState.CMoveTicks[0] > MOVE_TICK_THRESHOLD)
		{
			titleMenu.moveCursor(DOWN);
			inputState.CMoveTicks[0] = titleMenu.getNumberOfTicks();
		}
	}

	if (inputState.CJoyY[0] == 0 && inputState.CUp == false && inputState.CDown == false)
				inputState.CMoveTicks[0] = -MOVE_TICK_THRESHOLD; //If direction is not pressed down, the game should respond immediately when it is pressed down
}

void processOptionsScreenInput(InputState &inputState, GameState &gameState, Menu &optionsMenu, pvr_ptr_t &titleScreenImage, pvr_ptr_t &optionsScreenImage, pvr_ptr_t &cursorImage, pvr_ptr_t &gameOverImage, pvr_ptr_t &backgroundImage, pvr_ptr_t &creditsImage, pvr_ptr_t &winImage, sfxhnd_t &laserSound, std::vector<pvr_ptr_t> &storyImages, std::map<const std::string, pvr_ptr_t > &images, std::vector<Player> &players, std::vector<Laser> &lasers, std::vector<Enemy> &enemies, std::vector<Level> &levels, std::vector<Path> &paths)
{
	//In case the number of ticks overflows
	if (optionsMenu.getNumberOfTicks() - inputState.CMoveTicks[0] < 0)
		inputState.CMoveTicks[0] = optionsMenu.getNumberOfTicks();
	if (optionsMenu.getNumberOfTicks() - inputState.CATicks[0] < 0)
		inputState.CATicks[0] = optionsMenu.getNumberOfTicks();

	if (inputState.CA[0])
	{
		if (optionsMenu.getNumberOfTicks() - inputState.CATicks[0] > A_BUTTON_TICK_THRESHOLD)
		{
			switch (optionsMenu.getCursorIndex())
			{
			case ONE_PLAYER_ENTRY:
				gameState.setNumberOfPlayers(1);
				gameState.setScreen(TITLE_SCREEN, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, inputState, images, players, lasers, enemies, levels, paths);
				break;
			case TWO_PLAYER_ENTRY:
				gameState.setNumberOfPlayers(2);
				gameState.setScreen(TITLE_SCREEN, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, inputState, images, players, lasers, enemies, levels, paths);
				break;
			case THREE_PLAYER_ENTRY:
				gameState.setNumberOfPlayers(3);
				gameState.setScreen(TITLE_SCREEN, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, inputState, images, players, lasers, enemies, levels, paths);
				break;
			case FOUR_PLAYER_ENTRY:
				gameState.setNumberOfPlayers(4);
				gameState.setScreen(TITLE_SCREEN, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, inputState, images, players, lasers, enemies, levels, paths);
				break;
			}
			inputState.CATicks[0] = optionsMenu.getNumberOfTicks();
		}
	}
	else
		inputState.CATicks[0] = -A_BUTTON_TICK_THRESHOLD; //If A is not pressed down, the game should respond immediately when it is pressed down


	if (inputState.CUp[0])
	{
		if (optionsMenu.getNumberOfTicks() - inputState.CMoveTicks[0] > MOVE_TICK_THRESHOLD)
		{
			optionsMenu.moveCursor(UP);
			inputState.CMoveTicks[0] = optionsMenu.getNumberOfTicks();
		}
	}
	else if (inputState.CDown[0])
	{
		if (optionsMenu.getNumberOfTicks() - inputState.CMoveTicks[0] > MOVE_TICK_THRESHOLD)
		{
			optionsMenu.moveCursor(DOWN);
			inputState.CMoveTicks[0] = optionsMenu.getNumberOfTicks();
		}
	}

	if (inputState.CJoyY[0] < 0)
	{
		if (optionsMenu.getNumberOfTicks() - inputState.CMoveTicks[0] > MOVE_TICK_THRESHOLD)
		{
			optionsMenu.moveCursor(UP);
			inputState.CMoveTicks[0] = optionsMenu.getNumberOfTicks();
		}
	}

	else if (inputState.CJoyY[0] > 0)
	{
		if (optionsMenu.getNumberOfTicks() - inputState.CMoveTicks[0] > MOVE_TICK_THRESHOLD)
		{
			optionsMenu.moveCursor(DOWN);
			inputState.CMoveTicks[0] = optionsMenu.getNumberOfTicks();
		}
	}

	if (inputState.CJoyY[0] == 0 && inputState.CUp == false && inputState.CDown == false)
		inputState.CMoveTicks[0] = -MOVE_TICK_THRESHOLD; //If direction is not pressed down, the game should respond immediately when it is pressed down
}

void processCreditsScreenInput(InputState &inputState, GameState &gameState, Menu &creditsMenu, pvr_ptr_t &titleScreenImage, pvr_ptr_t &optionsScreenImage, pvr_ptr_t &cursorImage, pvr_ptr_t &gameOverImage, pvr_ptr_t &backgroundImage, pvr_ptr_t &creditsImage, pvr_ptr_t &winImage, sfxhnd_t &laserSound, std::vector<pvr_ptr_t> &storyImages, std::map<const std::string, pvr_ptr_t > &images, std::vector<Player> &players, std::vector<Laser> &lasers, std::vector<Enemy> &enemies, std::vector<Level> &levels, std::vector<Path> &paths)
{
	//In case the number of ticks overflows
	if (creditsMenu.getNumberOfTicks() - inputState.CATicks[0] < 0)
		inputState.CATicks[0] = creditsMenu.getNumberOfTicks();

	if (inputState.CA[0])
	{
		if (creditsMenu.getNumberOfTicks() - inputState.CATicks[0] > A_BUTTON_TICK_THRESHOLD)
		{
			gameState.setScreen(TITLE_SCREEN, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, inputState, images, players, lasers, enemies, levels, paths);
			inputState.CATicks[0] = creditsMenu.getNumberOfTicks();
		}
	}
	else
		inputState.CATicks[0] = -A_BUTTON_TICK_THRESHOLD; //If A is not pressed down, the game should respond immediately when it is pressed down
}

void processStoryInput(InputState &inputState, GameState &gameState, StoryManager &storyManager, pvr_ptr_t &titleScreenImage, pvr_ptr_t &optionsScreenImage, pvr_ptr_t &cursorImage, pvr_ptr_t &gameOverImage, pvr_ptr_t &backgroundImage, pvr_ptr_t &creditsImage, pvr_ptr_t &winImage, sfxhnd_t &laserSound, std::vector<pvr_ptr_t> &storyImages, std::map<const std::string, pvr_ptr_t > &images, std::vector<Player> &players, std::vector<Laser> &lasers, std::vector<Enemy> &enemies, std::vector<Level> &levels, std::vector<Path> &paths)
{
	for (int i = 0; i < gameState.getNumberOfPlayers(); ++i)
	{
		if (inputState.CA[i])
		{
			if (inputState.CAReady[i])
			{
				if (storyManager.index < STORY_IMAGES_ON_LEVEL[gameState.getCurrentLevel()])
				{
					++storyManager.index;
					inputState.CAReady[i] = false;
				}
				if (storyManager.index >= STORY_IMAGES_ON_LEVEL[gameState.getCurrentLevel()])
					gameState.setScreen(GAME_SCREEN, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, inputState, images, players, lasers, enemies, levels, paths);
			}
		}
		else
			inputState.CAReady[i] = true;
	}
}

void processInput(int numberOfTicks, InputState &inputState, GameState &gameState, Menu &optionsMenu, pvr_ptr_t &titleScreenImage, pvr_ptr_t &optionsScreenImage, pvr_ptr_t &cursorImage, pvr_ptr_t &gameOverImage, pvr_ptr_t &backgroundImage, pvr_ptr_t &creditsImage, pvr_ptr_t &winImage, sfxhnd_t &laserSound, std::vector<pvr_ptr_t> &storyImages, std::map<const std::string, pvr_ptr_t > &images, std::vector<Player> &players, std::vector<Laser> &lasers, std::vector<Enemy> &enemies, std::vector<Level> &levels, std::vector<Path> &paths)
{
	for (int i = 0; i < gameState.getNumberOfPlayers() ; ++i)
	{
		//In case the number of ticks overflows
		if (numberOfTicks - inputState.CLeftFireTicks[i] < 0)
			inputState.CLeftFireTicks[i] = numberOfTicks;

		if (numberOfTicks - inputState.CRightFireTicks[i] < 0)
					inputState.CRightFireTicks[i] = numberOfTicks;

		if (gameState.getPaused())
		{
			if (inputState.CStart[i])
			{
				if (inputState.CStartReady[i])
				{
					gameState.setPaused(false);
					inputState.CStartReady[i] = false;
					sndoggvorbis_volume(SOUND_HIGH_VOLUME);
				}
			}
			else
				inputState.CStartReady[i] = true;

			if (inputState.CA[i] && inputState.CB[i])
			{
				sndoggvorbis_volume(SOUND_HIGH_VOLUME);
				gameState.setScreen(TITLE_SCREEN, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, inputState, images, players, lasers, enemies, levels, paths);
			}

			continue;
		}
		if (inputState.CUp[i])
		{
			players[i].move(0, -PLAYER_VELOCITY_Y, enemies, lasers);
		}
		else if (inputState.CDown[i])
		{
			players[i].move(0, PLAYER_VELOCITY_Y, enemies, lasers);
		}

		if (inputState.CLeft[i])
		{
			players[i].move(-PLAYER_VELOCITY_X, 0, enemies, lasers);
		}
		else if (inputState.CRight[i])
		{
			players[i].move(PLAYER_VELOCITY_X, 0, enemies, lasers);
		}

		if (inputState.CJoyY[i] != 0)
		{
			players[i].move(0, inputState.CJoyY[i], enemies, lasers);
		}

		if (inputState.CJoyX[i] != 0)
		{
			players[i].move(inputState.CJoyX[i], 0, enemies, lasers);
		}

		if (inputState.CLeftTrigger[i])
		{
			if (inputState.CLeftFireReady[i])
			{
				if (numberOfTicks - inputState.CLeftFireTicks[i] > FIRE_TICK_THRESHOLD)
				{
					players[i].shoot(0, -LASER_VELOCITY_Y, LEFT_GUN, lasers, laserSound);
					inputState.CLeftFireReady[i] = false;
					inputState.CLeftFireTicks[i] = numberOfTicks;
				}
			}
		}
		else
			inputState.CLeftFireReady[i] = true;

		if (inputState.CRightTrigger[i])
		{
			if (inputState.CRightFireReady[i])
			{
				if (numberOfTicks - inputState.CRightFireTicks[i] > FIRE_TICK_THRESHOLD)
				{
					players[i].shoot(0, -LASER_VELOCITY_Y, RIGHT_GUN, lasers, laserSound);
					inputState.CRightFireReady[i] = false;
					inputState.CRightFireTicks[i] = numberOfTicks;
				}
			}
		}
		else
			inputState.CRightFireReady[i] = true;

		if (inputState.CStart[i])
		{
			if (inputState.CStartReady[i])
			{
				gameState.setPaused(true);
				inputState.CStartReady[i] = false;
				sndoggvorbis_volume(SOUND_LOW_VOLUME);
			}
		}
		else
			inputState.CStartReady[i] = true;
	}
}

void resetNumberOfTicks(InputState *inputState)
{
	for (int i = 0; i < MAX_PLAYERS; ++i)
	{
		inputState->CLeftFireTicks[i] = 0;
		inputState->CRightFireTicks[i] = 0;
		inputState->CMoveTicks[i] = 0;
		inputState->CATicks[i] = 0;
	}
}
