#include "../HeaderFiles/Level.h"
#include "../HeaderFiles/Utility.h"

void createSurvival(std::vector<Level> &levels)
{
	std::vector<int> ticksUntilNewEnemyRow;
	ticksUntilNewEnemyRow.push_back(LEVEL_START_TICKS + 1);
	ticksUntilNewEnemyRow.push_back(LEVEL_START_TICKS + 500);
	ticksUntilNewEnemyRow.push_back(LEVEL_START_TICKS + 1000);
	ticksUntilNewEnemyRow.push_back(LEVEL_START_TICKS + 1500);
	ticksUntilNewEnemyRow.push_back(LEVEL_START_TICKS + 2000);
	Level level(0, 8, 5, 0, 2000, ticksUntilNewEnemyRow);
	levels.push_back(level);
}

void restartSurvivalLevel(std::vector<Level> &levels)
{
	levels.clear();
	createSurvival(levels);
}

void createLevels(std::vector<Level> &levels)
{
	std::vector<int> ticksUntilNewEnemyRow;
	ticksUntilNewEnemyRow.push_back(LEVEL_START_TICKS + 1);
	Level level(0, 8, 1, 0, 10000, ticksUntilNewEnemyRow);
	levels.push_back(level);

	ticksUntilNewEnemyRow.push_back(LEVEL_START_TICKS + 500);
	Level level2(0, 8, 2, 0, 8000, ticksUntilNewEnemyRow);
	levels.push_back(level2);

	ticksUntilNewEnemyRow.push_back(LEVEL_START_TICKS + 1000);
	Level level3(0, 8, 3, 0, 6000, ticksUntilNewEnemyRow);
	levels.push_back(level3);

	ticksUntilNewEnemyRow.push_back(LEVEL_START_TICKS + 1500);
	Level level4(0, 8, 4, 0, 4000, ticksUntilNewEnemyRow);
	levels.push_back(level4);

	ticksUntilNewEnemyRow.push_back(LEVEL_START_TICKS + 2000);
	Level level5(0, 8, 5, 0, 3500, ticksUntilNewEnemyRow);
	levels.push_back(level5);

	Level level6(0, 8, 5, 0, 3000, ticksUntilNewEnemyRow);
	levels.push_back(level6);

	Level level7(0, 8, 5, 0, 2500, ticksUntilNewEnemyRow);
	levels.push_back(level7);

	Level level8(0, 8, 5, 0, 2000, ticksUntilNewEnemyRow);
	levels.push_back(level8);

	//Ending
	Level level9(0, 8, 0, 0, 2000, ticksUntilNewEnemyRow);
	levels.push_back(level9);
}

Level::Level(int initialNumberOfTicks, int initialTicksPerEnemy, int initialNumberOfEnemyRows, float initialEnemyFleetXCoordinate, int initialChanceOfLaser, const std::vector<int> &initialTicksUntilNewEnemyRow)
: numberOfTicks(initialNumberOfTicks),
  ticksPerEnemy(initialTicksPerEnemy),
  numberOfEnemyRows(initialNumberOfEnemyRows),
  ticksUntilNewEnemyRow(initialTicksUntilNewEnemyRow),
  enemyFleetXCoordinate(initialEnemyFleetXCoordinate),
  addNewRow(false),
  currentRow(0),
  enemyFleetXVelocity(ENEMY_FLEET_X_VELOCITY),
  chanceOfLaser(initialChanceOfLaser),
  levelStarted(false)
{
}

void Level::run(std::vector<Player> &players, std::vector<Enemy> &enemies, std::vector<Laser> &lasers, std::vector<Path> &paths, sfxhnd_t &laserSound)
{
	//Just in case numberOfTicks overflows
	if (numberOfTicks < 0)
		numberOfTicks = 0;

	if (numberOfTicks < LEVEL_START_TICKS)
	{
		++numberOfTicks;
	}
	else
	{
		levelStarted = true;
		if (numberOfTicks % ticksUntilNewEnemyRow[currentRow] == 0 && currentRow < numberOfEnemyRows)
		{
			addNewRow = true;
			++currentRow;
			int tmpSize = paths.size();
			if (tmpSize == 0)
				currentPathIndex = 0;
			else
				currentPathIndex = rand() % tmpSize;
		}
		if (numberOfTicks % ticksPerEnemy == 0 && (enemies.size() % ENEMIES_PER_ROW != 0 || addNewRow == true))
		{
			addNewRow = false;
			Point startingPoint = paths[currentPathIndex].getPoint(0);
			enemies.push_back(Enemy(IMAGE_PATH_UFO, IMAGE_WIDTH_UFO, IMAGE_HEIGHT_UFO, startingPoint.x,
					startingPoint.y, ENEMY_STARTING_Z + enemies.size(), SPRITE_ENEMY_WIDTH,
					SPRITE_ENEMY_HEIGHT));
		}

		for (int i = 0; i < (signed) enemies.size(); ++i)
		{

			if (!enemies[i].finishedPath(paths[currentPathIndex]))
				enemies[i].moveOnPath(paths[currentPathIndex], players, lasers);
			else
			{
				//Randomly shoot lasers
				if (rand() % chanceOfLaser == 0)
					enemies[i].shoot(0, ENEMY_LASER_VELOCITY_Y, lasers, laserSound);
				//move them to formation if they've completed the path
				enemies[i].moveToPoint(Point(enemyFleetXCoordinate + (i % ENEMIES_PER_ROW) *
						(SPRITE_ENEMY_WIDTH + ENEMY_PADDING),
						TOP_BOUNDARY_Y + (i / ENEMIES_PER_ROW) * (SPRITE_ENEMY_HEIGHT + ENEMY_PADDING)), players, lasers);
			}
		}

		//Move fleet left or right
		if (enemyFleetXCoordinate <= LEFT_BOUNDARY_X)
		{
			enemyFleetXVelocity = -enemyFleetXVelocity;
			enemyFleetXCoordinate = LEFT_BOUNDARY_X;
		}
		else if(enemyFleetXCoordinate + ENEMIES_PER_ROW * (SPRITE_ENEMY_WIDTH + ENEMY_PADDING)
				>= RIGHT_BOUNDARY_X)
		{
			enemyFleetXVelocity = -enemyFleetXVelocity;
			enemyFleetXCoordinate = RIGHT_BOUNDARY_X - ENEMIES_PER_ROW * (SPRITE_ENEMY_WIDTH + ENEMY_PADDING);
		}

		enemyFleetXCoordinate += enemyFleetXVelocity;

		++numberOfTicks;
	}
}

void Level::setEnemyFleetXCoordinate(float newEnemyFleetXCoordinate)
{
	enemyFleetXCoordinate = newEnemyFleetXCoordinate;
}

bool Level::levelComplete(std::vector<Player> &players, std::vector<Enemy> &enemies)
{
	//Was in levelComplete() but that caused crash
	for (int i = 0; i < (signed) enemies.size(); ++i)
	{
		if (currentRow < numberOfEnemyRows)
			return false;

		if (!enemies[i].getDead())
			return false;
	}

	if (enemies.size() == 0)
		return false;

	cleanUp(players, enemies);
	return true;
}

void Level::cleanUp(std::vector<Player> &players, std::vector<Enemy> &enemies)
{
	enemies.clear();

	players[0].setCoordinates(PLAYER_SHIP1_STARTING_X, PLAYER_SHIP1_STARTING_Y, PLAYER_SHIP1_STARTING_Z);

	if (players.size() > 1)
		players[1].setCoordinates(PLAYER_SHIP2_STARTING_X, PLAYER_SHIP2_STARTING_Y, PLAYER_SHIP2_STARTING_Z);

	if (players.size() > 2)
		players[2].setCoordinates(PLAYER_SHIP3_STARTING_X, PLAYER_SHIP3_STARTING_Y, PLAYER_SHIP3_STARTING_Z);

	if (players.size() > 3)
			players[3].setCoordinates(PLAYER_SHIP4_STARTING_X, PLAYER_SHIP4_STARTING_Y, PLAYER_SHIP4_STARTING_Z);
}

bool Level::getLevelStarted() const
{
	return levelStarted;
}

int Level::getNumberOfTicks() const
{
	return numberOfTicks;
}
