
/*
 * Hidden Cosmos By John Miner
 *
 * Copyright 2008
 *
 */

#include <ctime>
#include <cstdlib>
#include <kos.h>
#include <arch/rtc.h>
#include <oggvorbis/sndoggvorbis.h>
#include <dc/sound/sfxmgr.h>
#include <map>
#include <vector>
#include <string>
#include "../HeaderFiles/Video.h"
#include "../HeaderFiles/Globals.h"
#include "../HeaderFiles/InputState.h"
#include "../HeaderFiles/Input.h"
#include "../HeaderFiles/DrawGame.h"
#include "../HeaderFiles/Utility.h"
#include "../HeaderFiles/CollisionDetection.h"
#include "../HeaderFiles/Movement.h"
#include "../HeaderFiles/Sprite.h"
#include "../HeaderFiles/Player.h"
#include "../HeaderFiles/Laser.h"
#include "../HeaderFiles/Enemy.h"
#include "../HeaderFiles/Path.h"
#include "../HeaderFiles/Level.h"
#include "../HeaderFiles/Menu.h"
#include "../HeaderFiles/Init.h"
#include "../HeaderFiles/GameState.h"
#include "../HeaderFiles/StoryManager.h"

extern uint8 romdisk_boot[];
KOS_INIT_ROMDISK(romdisk_boot);

int main(int argc, char *argv[])
{
	pvr_ptr_t titleScreenImage;
	pvr_ptr_t optionsScreenImage;
	pvr_ptr_t cursorImage;
	pvr_ptr_t gameOverImage;
	pvr_ptr_t backgroundImage;
	pvr_ptr_t creditsImage;
	pvr_ptr_t winImage;
	sfxhnd_t laserSound;
	std::vector<pvr_ptr_t> storyImages;
	std::map<const std::string, pvr_ptr_t> images;
	std::vector<Player> players;
	std::vector<Laser> lasers;
	std::vector<Enemy> enemies;
	std::vector<Level> levels;
	std::vector<Path> paths;
	GameState gameState;
	InputState inputState;
	Menu titleMenu(TITLE_SCREEN_ENTRIES);
	Menu optionsMenu(OPTIONS_SCREEN_ENTRIES);
	Menu creditsMenu(0); //No entries on the credits menu, it's basically just a screen
	StoryManager storyManager;
	int gameOverTicks = 0;
	int winTicks = 0;

	pvr_init_defaults();
	snd_stream_init();
	sndoggvorbis_init();
	srand(rtc_boot_time());

	gameState.setScreen(TITLE_SCREEN, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, inputState, images, players, lasers, enemies, levels, paths);

	while(true)
	{
		switch (gameState.getScreen())
		{
		case TITLE_SCREEN:
			getInput(&inputState, &gameState);
			processTitleScreenInput(inputState, gameState, titleMenu, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, images, players, lasers, enemies, levels, paths);
			if (gameState.getStateChanged())
			{
				titleMenu.resetNumberOfTicks();
				resetNumberOfTicks(&inputState);
				break;
			}
			titleMenu.incrementNumberOfTicks();
			drawTitleMenu(titleScreenImage, cursorImage, titleMenu);
			break;
		case OPTIONS_SCREEN:
			getInput(&inputState, &gameState);
			processOptionsScreenInput(inputState, gameState, optionsMenu, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, images, players, lasers, enemies, levels, paths);
			if (gameState.getStateChanged())
			{
				optionsMenu.resetNumberOfTicks();
				resetNumberOfTicks(&inputState);
				break;
			}
			optionsMenu.incrementNumberOfTicks();
			drawOptionsMenu(optionsScreenImage, cursorImage, optionsMenu);
			break;
		case CREDITS_SCREEN:
			getInput(&inputState, &gameState);
			processCreditsScreenInput(inputState, gameState, creditsMenu, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, images, players, lasers, enemies, levels, paths);
			if (gameState.getStateChanged())
			{
				creditsMenu.resetNumberOfTicks();
				resetNumberOfTicks(&inputState);
				break;
			}
			creditsMenu.incrementNumberOfTicks();
			drawCreditsMenu(creditsImage);
			break;
		case STORY_SCREEN:
			getInput(&inputState, &gameState);
			processStoryInput(inputState, gameState, storyManager, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, images, players, lasers, enemies, levels, paths);
			if (gameState.getStateChanged())
			{
				storyManager.index = 0;
				break;
			}
			drawStory(storyImages, storyManager.index);
			break;
		case GAME_SCREEN:
			if (levels[gameState.getCurrentLevel()].getLevelStarted())
			{
				getInput(&inputState, &gameState);
				processInput(levels[gameState.getCurrentLevel()].getNumberOfTicks(), inputState, gameState, optionsMenu, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, images, players, lasers, enemies, levels, paths);

				if (gameState.getStateChanged())
					break;
			}
			if (!gameState.getPaused())
			{
				for (int i = 0; i < gameState.getNumberOfPlayers(); ++i)
					players[i].incrementInvincibilityTicks();

				moveLasers(players, enemies, lasers);
				levels[gameState.getCurrentLevel()].run(players, enemies, lasers, paths, laserSound);
				if (levels[gameState.getCurrentLevel()].levelComplete(players, enemies))
				{
					resetNumberOfTicks(&inputState);
					gameState.nextLevel();
					int index = gameState.getCurrentLevel();

					freeImage(&backgroundImage);
					loadImage(&backgroundImage, IMAGE_PATH_BACKGROUND[index], IMAGE_WIDTH_BACKGROUND[index], IMAGE_HEIGHT_BACKGROUND[index]);

					if (!backgroundImage)
						exit(EXIT_FAILURE);

					gameState.setScreen(STORY_SCREEN, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, inputState, images, players, lasers, enemies, levels, paths);
					break;
				}
			}
			if (checkForGameOver(players))
			{
				gameState.setScreen(GAME_OVER_SCREEN, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, inputState, images, players, lasers, enemies, levels, paths);
				break;
			}
			if (gameState.getCurrentLevel() >= WIN_LEVEL)
			{
				gameState.setScreen(WIN_SCREEN, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, inputState, images, players, lasers, enemies, levels, paths);
					break;
			}
			drawGame(gameState, backgroundImage, gameState.getCurrentLevel(), images, players, enemies, lasers, levels);
			break;
		case SURVIVAL_SCREEN:
			if (levels[gameState.getCurrentLevel()].getLevelStarted())
			{
				getInput(&inputState, &gameState);
				processInput(levels[gameState.getCurrentLevel()].getNumberOfTicks(), inputState, gameState, optionsMenu, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, images, players, lasers, enemies, levels, paths);
				if (gameState.getStateChanged())
					break;
			}
			if (!gameState.getPaused())
			{
				for (int i = 0; i < gameState.getNumberOfPlayers(); ++i)
					players[i].incrementInvincibilityTicks();

				moveLasers(players, enemies, lasers);
				levels[gameState.getCurrentLevel()].run(players, enemies, lasers, paths, laserSound);
				if (levels[gameState.getCurrentLevel()].levelComplete(players, enemies))
				{
					resetNumberOfTicks(&inputState);
					restartSurvivalLevel(levels);
				}
			}
			if (checkForGameOver(players))
			{
				gameState.setScreen(GAME_OVER_SCREEN, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, inputState, images, players, lasers, enemies, levels, paths);
				break;
			}
			//Magic number below is the index to the background image. I want the first image coordinates and dimensions
			drawGame(gameState, backgroundImage, 0, images, players, enemies, lasers, levels);
			break;
		case GAME_OVER_SCREEN:
			++gameOverTicks;

			if (gameOverTicks > GAME_OVER_TICK_THRESHOLD)
			{
				gameOverTicks = 0;
				gameState.setScreen(TITLE_SCREEN, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, inputState, images, players, lasers, enemies, levels, paths);
			}
			drawGameOver(gameOverImage);
			break;
		case WIN_SCREEN:
			++winTicks;

			if (winTicks > WIN_TICK_THRESHOLD)
			{
				winTicks = 0;
				gameState.setScreen(TITLE_SCREEN, titleScreenImage, optionsScreenImage, cursorImage, gameOverImage, backgroundImage, creditsImage, winImage, laserSound, storyImages, inputState, images, players, lasers, enemies, levels, paths);
			}
			drawWin(winImage);
			break;
		}
	}

	sndoggvorbis_stop();
	sndoggvorbis_shutdown();
	snd_stream_shutdown();

	return EXIT_SUCCESS;
}
