#include "../HeaderFiles/Player.h"
#include "../HeaderFiles/Utility.h"
#include "../HeaderFiles/Sprite.h"
#include "../HeaderFiles/Enemy.h"
#include "../HeaderFiles/Laser.h"

bool checkForGameOver(std::vector<Player> players)
{
	for (std::vector<Player>::iterator it = players.begin();
			it != players.end(); ++it)
	{
		if (it->getLives() > 0)
			return false;
	}

	return true;
}

Player::Player()
{
}

Player::Player(std::string initialImagePath, float initialImageWidth, float initialImageHeight, float initalX, float initialY, float initialZ, float initialWidth, float initialHeight, int initialLives) :
Sprite(initialImagePath, initialImageWidth, initialImageHeight, initalX,  initialY, initialZ, initialWidth, initialHeight),
lives(initialLives),
invincibilityTicks(0)
{
}

void Player::move(float xVelocity, float yVelocity, std::vector<Enemy> &enemies, std::vector<Laser> &lasers)
{
	if (dead)
		return;

	x += xVelocity;

	if (checkWallCollisions() == IS_COLLISION)
		x -= xVelocity;

	y += yVelocity;

	if (checkWallCollisions() == IS_COLLISION)
			y -= yVelocity;

	if (invincible())
		return;

	int enemyCollisionsIndex = checkSpriteCollisions(enemies);
	if (enemyCollisionsIndex != NO_SPRITE)
	{
		enemies[enemyCollisionsIndex].setDead(true);
		setDead(true);
	}

	//Special case with lasers because player originated lasers should not kill the player
	int laserCollisionsIndex = checkLaserCollisions(PLAYER_LASER, lasers);
	if (laserCollisionsIndex != NO_SPRITE)
	{
		lasers[laserCollisionsIndex].setDead(true);
		setDead(true);
	}
}

void Player::shoot(float xVelocity, float yVelocity, bool left, std::vector<Laser> &lasers, sfxhnd_t &laserSound)
{
	if (dead)
		return;

	snd_sfx_play(laserSound, 255, 128); //Max volume no pan

	bool laserAdded = false;

	for (std::vector<Laser>::iterator it = lasers.begin();
			it != lasers.end(); ++it)
	{
		if(it->getDead() == true)
		{
			if (left)
				it->setCoordinates(x + LASER_OFFSET_X_1, y - LASER_OFFSET_Y, it->getZ());
			else
				it->setCoordinates(x + LASER_OFFSET_X_2, y - LASER_OFFSET_Y, it->getZ());

			it->setDead(false);
			it->setVelocities(xVelocity, yVelocity);
			it->setPlayerLaser(PLAYER_LASER);
			laserAdded = true;
			break;
		}
	}

	if (laserAdded == false)
	{
		char index[32];

		integerToString(lasers.size(), index);

		if (left)
			lasers.push_back(Laser(IMAGE_PATH_LASER, IMAGE_WIDTH_LASER, IMAGE_HEIGHT_LASER, x + LASER_OFFSET_X_1, y - LASER_OFFSET_Y, LASER_STARTING_Z + lasers.size(), SPRITE_LASER_WIDTH, SPRITE_LASER_HEIGHT, xVelocity, yVelocity, PLAYER_LASER));
		else
			lasers.push_back(Laser(IMAGE_PATH_LASER, IMAGE_WIDTH_LASER, IMAGE_HEIGHT_LASER, x + LASER_OFFSET_X_2, y - LASER_OFFSET_Y, LASER_STARTING_Z + lasers.size(), SPRITE_LASER_WIDTH, SPRITE_LASER_HEIGHT, xVelocity, yVelocity, PLAYER_LASER));

		laserAdded = true;
	}
}

int Player::getLives() const
{
	return lives;
}

void Player::setLives(int newLives)
{
	lives = newLives;
}

void Player::setDead(bool newDead)
{
	if (invincible())
		return;

	if (newDead == true)
	{
		--lives;
		resetCoordinates();
	}

	if (lives <= 0)
		dead = true;
}

void Player::resetCoordinates()
{
	x = PLAYER_SHIP_STARTING_X_GENERIC;
	y = PLAYER_SHIP_STARTING_Y_GENERIC;
	invincibilityTicks = 0;
}

void Player::incrementInvincibilityTicks()
{
	if (invincible())
		++invincibilityTicks;
}

bool Player::invincible()
{
	if (invincibilityTicks < INVINCIBILITY_TICK_THRESHOLD)
		return true;

	return false;
}
