#include "../HeaderFiles/Sprite.h"
#include "../HeaderFiles/Globals.h"

Sprite::Sprite()
{
}

Sprite::Sprite(std::string initialImagePath, float initialImageWidth, float initialImageHeight, float initialX, float initialY, float initialZ, float initialWidth, float initialHeight) :
imagePath(initialImagePath), imageWidth(initialImageWidth), imageHeight(initialImageHeight), x(initialX), y(initialY), z(initialZ), width(initialWidth), height(initialHeight), dead(false)
{
}

void Sprite::setImagePath(std::string newImagePath)
{
	imagePath = newImagePath;
}

std::string Sprite::getImagePath() const
{
	return imagePath;
}

void Sprite::setCoordinates(float newX, float newY, float newZ)
{
	x = newX;
	y = newY;
	z = newZ;
}

Sprite::Sprite(const Sprite& sp)
{
	imagePath = sp.imagePath;
	x = sp.x;
	y = sp.y;
	z = sp.z;
	width = sp.width;
	height = sp.height;
	dead = sp.dead;
	imageWidth = sp.imageWidth;
	imageHeight = sp.imageHeight;
}

void Sprite::getCoordinates(float &curX, float &curY, float &curZ) const
{
	curX = x;
	curY = y;
	curZ = z;
}

float Sprite::getZ() const
{
	return z;
}

void Sprite::setDimensions(float newWidth, float newHeight)
{
	width = newWidth;
	height = newHeight;
}

void Sprite::getDimensions(float &curWidth, float &curHeight) const
{
	curWidth = width;
	curHeight = height;
}

void Sprite::getImageDimensions(float &curImageWidth, float &curImageHeight) const
{
	curImageWidth = imageWidth;
	curImageHeight = imageHeight;
}

bool Sprite::checkWallCollisions()
{
	if (RectangularCollisionDetection(
				BoundingBox(x, y, x + width, y + height),
				BoundingBox(TOP_BOUNDARY_X, TOP_BOUNDARY_Y,
						TOP_BOUNDARY_X  + TOP_BOUNDARY_WIDTH,
						TOP_BOUNDARY_Y - TOP_BOUNDARY_HEIGHT)))
						return IS_COLLISION;

	if (RectangularCollisionDetection(
				BoundingBox(x, y, x + width, y + height),
				BoundingBox(BOTTOM_BOUNDARY_X, BOTTOM_BOUNDARY_Y,
						BOTTOM_BOUNDARY_X + BOTTOM_BOUNDARY_WIDTH,
						BOTTOM_BOUNDARY_Y + BOTTOM_BOUNDARY_HEIGHT)))
						return IS_COLLISION;

	if (RectangularCollisionDetection(
		BoundingBox(x, y, x + width, y + height),
		BoundingBox(LEFT_BOUNDARY_X, LEFT_BOUNDARY_Y,
				LEFT_BOUNDARY_X - LEFT_BOUNDARY_WIDTH,
				LEFT_BOUNDARY_Y + LEFT_BOUNDARY_HEIGHT)))
				return IS_COLLISION;

	if (RectangularCollisionDetection(
			BoundingBox(x, y, x + width, y + height),
			BoundingBox(RIGHT_BOUNDARY_X, RIGHT_BOUNDARY_Y,
					RIGHT_BOUNDARY_X + RIGHT_BOUNDARY_WIDTH,
					RIGHT_BOUNDARY_Y + RIGHT_BOUNDARY_HEIGHT)))
					return IS_COLLISION;

	return !IS_COLLISION;
}

void Sprite::setDead(bool newRemove)
{
	dead = newRemove;
}

bool Sprite::getDead() const
{
	return dead;
}

bool Sprite::operator==(const Sprite& sp) const
{
	if (imagePath == sp.imagePath && x == sp.x && y == sp.y && z == sp.z && width == sp.width && height == sp.height)
		return true;

	return false;
}
